% % This code simulates a market for entrepreneurs and workers with double
% % adverse selection and learning. Its base version (i.e. without learning)
% % is meant to mimic the outcomes predicted in Boadway and Sato (2011).
% %
% % The code was tested on Matlab (R2015a) and GNU Octave (4.0.3).
% % To improve reproducibility, this version complies with Octave.
% %
% % Author: Diego d'Andria
% % This version: February 2017

% % The code is made of three files:
% % - model_main: this file is the one to be run and where the initial parameters
% %               are set.
% % - model_internal: this file contains the actual simulation code.
% % - Bayesian_learning: this is a function called by model_internal when learning
% %                      options are selected.


clc, clear all
more off;
% ------------------------------------------------------------------------


%% INITIAL SETTINGS

% SWITCHES
learning_type = 0;       % Type of learning:
% -1 - full information
% 0 - no learning (i.e. zero information)
% 1 - with learning


% MAIN PARAMETERS
replications = 50;       % no. of times the simulation is replicated
periods = 20;            % no. of periods the simulation is run
pop_workers = 1000;      % number of workers in the population
pop_entrepreneurs = 100; % no. of potential entrepreneurs in the population
R = 36.2;                % Productivity scaling
alpha = 0.7;             % Exponent in production function
rho = 2;                 % risk-free interest rate
alt_profit = 1;          % risk-free alternative income for entrepreneurs
n = 10;                  % no. of workers hired by each firm
learning_starts_at = 3;  % no. of initial periods where no learning occurs yet
learning_anoise = 0.15;  % variance of the noise in the entrepreneurs' quality signal
learning_qnoise = 0.25;  % variance of the noise in the workers' quality signal
interest_cap = 20;       % ceiling imposed to interest rates

cit_tax = 0;             % Tax or subsidy to profits (negative sign = tax)
pit_tax = 0.0;           % Tax or subsidy to labour costs (negative sign = tax)

pit_tax_incidence = 0.5; % If 1, 100% of the subsidy of the tax or subsidy is on the firm.
						 % If less than 1 (minimum is zero) this share of it goes to workers.


% Generate containers for results coming from each replication.
RESULTS_welfare_tot = zeros(replications, 1);
RESULTS_welfare_entr = zeros(replications, 1);
RESULTS_welfare_work = zeros(replications, 1);
RESULTS_welfare_gov = zeros(replications, 1);
RESULTS_entry_entr = zeros(replications, 1);
RESULTS_employed_work = zeros(replications, 1);


% Call the file "model_internal" each time a replication is run, and store values.
for rep=1:replications
    printf("Now simulating replication: ");
    disp(rep);
  run model_internal
end

printf("Replications completed. \n ");


% Take average values across replications:
AVGRESULTS_welfare_tot = mean(RESULTS_welfare_tot);
AVGRESULTS_welfare_entr = mean(RESULTS_welfare_entr);
AVGRESULTS_welfare_work = mean(RESULTS_welfare_work);
AVGRESULTS_welfare_gov = mean(RESULTS_welfare_gov);
AVGRESULTS_entry_entr = mean(RESULTS_entry_entr);
AVGRESULTS_employed_work = mean(RESULTS_employed_work);

%clearvars -except AVGRESULTS_*;  %MATLAB code
clear -x AVGRESULTS_*;  %Octave code

save ('results_saved');
%% END OF FILE
%%